################################################################################
############## This file contains useful functions for the paper ###############
################################################################################

library(sensitivitymv) # for truncated product
library(metap) # for Fisher's combination of p-values

hc.score <- function (pval) {
    ## this is a one-sided version modified from package fdrtool
    if (max(pval) > 1 | min(pval) < 0)
        stop("input p-values must all be in the range 0 to 1!")
    d = length(pval)
    F = rank(pval, ties.method = "max")/d
    v = F * (1 - F)/d
    v[v == 0] = min(v[v > 0])
    HC = (F - pval)/sqrt(v)
    return(HC)
}

find.trunc <- function(p, tau.seq = seq(0.9, 0.1, -0.05), width = 0.1, alpha = 0.01) {
    if (is.matrix(p)) {
        if (nrow(p) > 1) {
            return(apply(p, 1, find.trunc, tau.seq = tau.seq, width = width, alpha = alpha))
        }
    }
    tau.seq <- sort(tau.seq, decreasing = TRUE)
    n <- length(p)
    stopifnot(tau.seq[1] + width <= 1)
    f.hat <- sapply(tau.seq, function(x) sum(p > x & p < x + width)) / (n * width)
    F.hat <- sapply(tau.seq, function(x) sum(p <= x)) / n
    test.p <- sapply(1:length(tau.seq), function(k) binom.test(f.hat[k] * n * width, n, width * F.hat[k] / tau.seq[k], alternative = "greater")$p.value)
    ind <- min(suppressWarnings(min(which(test.p > alpha))), length(tau.seq))
    tau.seq[ind]
}

find.trunc.fisher <- function(p, tau.seq = seq(0.9, 0.1, -0.05), width = 0.1, alpha = 0.01) {
    if (is.matrix(p)) {
        if (nrow(p) > 1) {
            return(apply(p, 1, find.trunc.fisher, tau.seq = tau.seq, width = width, alpha = alpha))
        }
    }
    tau.seq <- sort(tau.seq, decreasing = TRUE)
    n <- length(p)
    stopifnot(tau.seq[1] + width <= 1)
    ## f.hat <- sapply(tau.seq, function(x) sum(p > x & p < x + width)) / (n * width)
    ## F.hat <- sapply(tau.seq, function(x) sum(p <= x)) / n
    test.p <- sapply(1:length(tau.seq), function(k) binom.test(sum(p > tau.seq[k] & p < tau.seq[k] + width), max(1, sum(p < tau.seq[k] + width)), width, alternative = "less")$p.value)
    ind <- min(suppressWarnings(min(which(test.p < alpha))),
               min(which(sapply(1:length(tau.seq), function(k) sum(p < tau.seq[k])) <= 10)),
               length(tau.seq))
    tau.seq[ind]
}

test.global.null <- function(p,
                             method = c("Bonferroni", "Sidak", "Fisher", "Tukey", "TruncatedP"),
                             scan = FALSE,
                             trunc = 1,
                             alpha0 = 0.5,
                             tukey.null = NULL,
                             nsim = 10000) {

    method <- match.arg(method, c("Bonferroni", "Sidak", "Fisher", "Tukey", "TruncatedP"))
    ## if (scan && (min(trunc) != 1)) {
    ##     stop("Either specify a truncation threshold or use the scan statistic. Cannot do both!")
    ## }
    if (is.vector(p)) {
        p <- t(as.matrix(p))
    }
    if (max(trunc) > 1) {
        stop("Truncation threshold cannot be greater than 1.")
    }

    if (method == "Tukey" && is.null(tukey.null)) {
        tukey.null <- sapply(1:nsim, function(sim) {
            p.values <- runif(ncol(p))
            max(hc.score(p.values)[p.values <= alpha0]) })
    }

    if (!scan && (min(trunc) == 1)) { ## classic method
        if (method == "Bonferroni") {
            return(apply(p, 1, function(p) min(min(p) * length(p), 1)))
        } else if (method == "Sidak") {
            return(apply(p, 1, function(p) 1 - (1 - min(p))^length(p)))
        } else if (method == "Fisher") {
            return(apply(p, 1, function(p) {
                if (length(p) >= 2) {
                    sumlog(p)$p
                } else {
                    p
                } }))
        } else if (method == "Tukey") {
            hc.stat <- apply(p, 1, function(p) max(hc.score(p)[p <= alpha0]))
            return(sapply(hc.stat, function(hc.stat) mean(tukey.null >= hc.stat)))
        } else if (method == "TruncatedP") {
            return(apply(p, 1, function(p) truncatedP(p)))
        }
    }

    if (length(trunc) == 1) {
        trunc <- rep(trunc, nrow(p))
    }
    if (min(trunc) < 1) {
        return(sapply(1:nrow(p), function(i) {
            pp <- p[i, ]
            pp <- pp[pp <= trunc[i]] / trunc[i]
            if (length(pp) == 0) {
                return(1)
            } else {
                return(test.global.null(pp, method, scan = FALSE, trunc = 1, alpha0, tukey.null, nsim = nsim))
            }
        }))
    }

    ## if (scan) {
    ##     stats <- apply(p, 1, function(p) min(sapply(unique(pmax(p, alpha0)), function(pp) test.global.null(p, method, scan = FALSE, trunc = pp - .Machine$double.eps, alpha0, tukey.null, nsim = nsim)))) # we use pp - eso, otherwise there is a bug when pp = 1
    ##     p.null <- matrix(runif(ncol(p) * nsim), nsim, ncol(p))
    ##     null.stats <- apply(p.null, 1, function(p) min(sapply(unique(pmax(p, alpha0)), function(pp) test.global.null(p, method, scan = FALSE, trunc = pp, alpha0, tukey.null, nsim = nsim))))
    ##     return(sapply(stats, function(stat) mean(null.stats <= stat)))
    ## }

}

## Data: x ~ N(mu, 1)
## Test if all mu <= 0.
test.union <- function(x, beta = 0.01) {

    p <- length(x)
    mu.upper <- pmin(x + qnorm(1 - beta / p), 0)
    min(sum(1 - pnorm(max(x) - mu.upper)) + beta, 1)

}
