#!/usr/bin/env Rscript

args <- commandArgs(trailingOnly=TRUE)
if (length(args) > 0) {
    nrun <- as.numeric(args[1])
} else {
    nrun <- 10 # the paper uses 10000
}

################################################################################
############## This R code generates Tables 2 to 4 in the paper ################
################################################################################

library(sensitivitymv) # for truncated product
library(metap) # for Fisher's combination of p-values
library(QualInt) # for IBGA and LRT tests

source("./core.R")

#### Table 2

settings <- c("all null",
              "one strong many null",
              "one strong many conservative",
              "one strong many very conservative",
              "one strong many extremely conservative",
              ## "some moderate many null",
              ## "some moderate many conservative",
              "many weak many null",
              "many weak many conservative",
              "many weak many very conservative")
## methods <- expand.grid(method = c("Bonferroni", "Fisher", "Tukey", "TruncatedP", "IBGA", "LRT"),
##                        scan = c(FALSE, TRUE),
##                        trunc = c(1, 0.5, "adaptive"))
methods <- expand.grid(method = c("Bonferroni", "Fisher", "Tukey", "TruncatedP"),
                       scan = c(FALSE, TRUE),
                       trunc = c(1, 0.5, 0.8, "adaptive"))
methods <- subset(methods, !scan)

n <- 100
## nrun <- 10 # the paper uses 10000

df <- data.frame()
power.union <- rep(0, length(settings))

for (i in 1:length(settings)) {
    setting <- settings[i]
    x <- matrix(rnorm(n * nrun), nrun, n)
    if (setting == "all null") {
        x <- x
    } else if (setting == "one strong many null") {
        x[, 1] <- x[, 1] + 4
    } else if (setting == "one strong many conservative") {
        x[, 1] <- x[, 1] + 4
        x[, -1] <- x[, -1] - 1
    } else if (setting == "one strong many very conservative") {
        x[, 1] <- x[, 1] + 4
        x[, -1] <- x[, -1] - 4
    } else if (setting == "one strong many extremely conservative") {
        x[, 1] <- x[, 1] + 4
        x[, -1] <- x[, -1] - 10
    } else if (setting == "some moderate many null") {
        x[, 1:5] <- x[, 1:5] + 2.5
    } else if (setting == "some moderate many conservative") {
        x[, 1:5] <- x[, 1:5] + 2.5
        x[, -(1:5)] <- x[, -(1:5)] - 1
    } else if (setting == "many weak many null") {
        x[, 1:20] <- x[, 1:20] + 1
    } else if (setting == "many weak many conservative") {
        x[, 1:20] <- x[, 1:20] + 1
        x[, -(1:20)] <- x[, -(1:20)] - 1
    } else if (setting == "many weak many very conservative") {
        x[, 1:20] <- x[, 1:20] + 1
        x[, -(1:20)] <- x[, -(1:20)] - 4
    }
    p.values <- 1 - pnorm(x)
    p.values <- pmax(p.values, .Machine$double.eps)
    for (j in 1:nrow(methods)) {
        if (methods$method[j] %in% c("IBGA", "LRT")) {
            combined.p.value <- sapply(1:nrun, function(k) qualval(x[k, ], rep(1, n), as.character(methods$method[j]))$pvalue)
        } else {
            if (methods$trunc[j] == "adaptive") {
                if (methods$method[j] %in% c("Bonferroni", "Tukey")) {
                    tau <- find.trunc(p.values, alpha = 0.05)
                } else {
                    tau <- find.trunc(p.values, alpha = 0.5)
                }
            } else {
                tau <- as.numeric(as.character(methods$trunc[j]))
            }
            combined.p.value <- test.global.null(p.values, as.character(methods$method[j]), methods$scan[j], tau, nsim = 10000)
        }
        power <- mean(combined.p.value < 0.05)
        df.add <- data.frame(setting, methods[j, ], power, row.names = NULL)
        ## print(df.add)
        df <- rbind(df, df.add)
    }
    pval.union <- sapply(1:nrun, function(k) test.union(x[k, ]))
    power.union[i] <- mean(pval.union < 0.05)
}

save(df, power.union, file = "df_new3.rda")
## load("df_new.rda") # adaptive is wrong

## library(xtable)
## print(xtable(df), include.rownames = FALSE)

df$combine <- factor(paste(df$scan, df$trunc))
levels(df$combine) <- c("0.5", "0.8", "1", "Adaptive")
df$combine <- factor(df$combine,
                     c("1", "0.8", "0.5", "Adaptive"))
df$power <- round(df$power * 100, 1)

library(tables)
tab <- tabular(setting * method  ~ combine * (power * Format(digits = 1)) * Heading() * identity, data = subset(df))
tab
## latex(tab)

power.union

#### Table 3

n <- 1000
## nrun <- 1000

## scb <- rep(0, nrun)
cb1 <- rep(0, nrun)
cb2 <- rep(0, nrun)
cba <- rep(0, nrun)
b <- rep(0, nrun)

x <- matrix(rnorm(n * nrun), nrun, n)
x[, 1:20] <- x[, 1:20] + 4
x[, -(1:20)] <- x[, -(1:20)]
p.values <- 1 - pnorm(x)
p.values <- pmax(p.values, .Machine$double.eps)

for (j in 1:nrun) {
    ## threshold <- scan.bonferroni.test(p.values[j, ], 0.5, cv[1:length(p.values[j, ])])
    ## scb[j] <- sum(p.values[j, ] <= threshold)
    cb1[j] <- sum(p.adjust(p.values[j, p.values[j, ] < 0.5]/0.5, "bonferroni") < 0.05)
    cb2[j] <- sum(p.adjust(p.values[j, p.values[j, ] < 0.8]/0.8, "bonferroni") < 0.05)
    tau <- find.trunc(p.values[j, ], alpha = 0.05)
    cba[j] <- sum(p.adjust(p.values[j, p.values[j, ] < tau]/tau, "bonferroni") < 0.05)
    b[j] <- sum(p.adjust(p.values[j, ], "bonferroni") < 0.05)
}

summary(b)
summary(cb2)
summary(cb1)
summary(cba)




##### Table 4

settings <- c(#"all null",
    "one positive other null",
    "one positive one negative",
    "one positive other negative",
    "some positive other negative",
    "half positive half negative",
    "gradual1",
    "gradual2")
methods <- expand.grid(method = c("Bonferroni", "Fisher", "Tukey", "TruncatedP", "IBGA", "LRT"),
                       scan = c(FALSE, TRUE),
                       trunc = c(1, 0.8, 0.5, "adaptive"))
## methods <- expand.grid(method = c("Bonferroni", "Fisher"),
##                        scan = c(FALSE, TRUE),
##                        trunc = c(1, 0.8, 0.5, "adaptive"))
## methods <- expand.grid(method = c("Bonferroni", "Fisher", "Tukey", "TruncatedP", "IBGA"),
##                        scan = c(FALSE, TRUE),
##                        trunc = c(1, 0.5, "adaptive"))
methods <- subset(methods, !scan)

n <- 100
## nrun <- 10 # the paper uses 10000

one.sim <- function(i) {
    setting <- settings[i]
    x <- matrix(rnorm(n * nrun), nrun, n)
    if (setting == "all null") {
        x <- x
    } else if (setting == "one positive other null") {
        x[, 1] <- x[, 1] + 4
    } else if (setting == "half positive half negative") {
        x[, 1:(n/2)] <- x[, 1:(n/2)] + 1
        x[, -(1:(n/2))] <- x[, -(1:(n/2))] - 1
    } else if (setting == "one positive one negative") {
        x[, 1] <- x[, 1] + 4
        x[, 2] <- x[, 2] - 4
    } else if (setting == "one positive other negative") {
        x[, 1] <- x[, 1] + 4
        x[, -1] <- x[, -1] - 1
    } else if (setting == "some positive other negative") {
        x[, 1:20] <- x[, 1:20] + 1
        x[, -(1:20)] <- x[, -(1:20)] - 1
    } else if (setting == "gradual1") {
        x <- t(t(x) + seq(-1.5, 2, length = n))
    } else if (setting == "gradual2") {
        x <- t(t(x) + seq(-1.5, 4, length = n))
    } else {
        stop("No such setting")
    }
    df <- data.frame()
    for (j in 1:nrow(methods)) {
        if (methods$method[j] == "LRT") {
            if (methods$trunc[j] == 1) {
                combined.p.value <- sapply(1:(nrun/10), function(k) qualval(x[k, ], rep(1, n), as.character(methods$method[j]))$pvalue)
            } else {
                combined.p.value <- rep(1, nrun/10)
            }
        } else if (methods$method[j] == "IBGA") {
            combined.p.value <- sapply(1:nrun, function(k) qualval(x[k, ], rep(1, n), as.character(methods$method[j]))$pvalue)
        } else {
            p.values <- 1 - pnorm(x)
            p.values <- pmax(p.values, .Machine$double.eps)
            if (methods$trunc[j] == "adaptive") {
                tau <- find.trunc(p.values, alpha = 0.5)
            } else {
                tau <- as.numeric(as.character(methods$trunc[j]))
            }
            combined.p.value1 <- test.global.null(p.values, as.character(methods$method[j]), methods$scan[j], tau, nsim = 10000)
            p.values <- pnorm(x)
            p.values <- pmax(p.values, .Machine$double.eps)
            if (methods$trunc[j] == "adaptive") {
                tau <- find.trunc(p.values, alpha = 0.5)
            } else {
                tau <- as.numeric(as.character(methods$trunc[j]))
            }
            combined.p.value2 <- test.global.null(p.values, as.character(methods$method[j]), methods$scan[j], tau, nsim = 10000)
            combined.p.value <- pmax(combined.p.value1, combined.p.value2)
        }
        power <- mean(combined.p.value < 0.05)
        df.add <- data.frame(setting, methods[j, ], power, row.names = NULL)
        ## print(df.add)
        df <- rbind(df, df.add)
    }
    df
}

library(parallel)
df <- mclapply(1:length(settings), function(i) one.sim(i), mc.cores = 4)
df <- do.call(rbind, df)

save(df, file = "df_qualint.rda")

load(file = "df_qualint.rda")

df$combine <- factor(paste(df$scan, df$trunc))
levels(df$combine) <- c("0.5", "0.8", "1", "Adaptive")
df$combine <- factor(df$combine,
                     c("1", "0.8", "0.5", "Adaptive"))
df$power <- round(df$power * 100, 1)

library(tables)
tab <- tabular(setting * method  ~ combine * (power * Format(digits = 1)) * Heading() * identity, data = subset(df))
tab

## latex(tab)
